@@ -83,6 +83,7 @@ struct _EBookBackendGALPrivate {
 
 	EBookBackendCache *cache;
 	gboolean marked_for_offline;
+	GMutex 		*ldap_lock;
 
 	/* our operations */
 	GStaticRecMutex op_hash_mutex;
@@ -137,6 +138,7 @@ struct prop_info {
 	STRING_PROP   (E_CONTACT_FULL_NAME,   "displayName" ),
 	STRING_PROP   (E_CONTACT_GIVEN_NAME,  "givenName" ),
 	STRING_PROP   (E_CONTACT_FAMILY_NAME, "sn" ),
+	STRING_PROP   (E_CONTACT_NICKNAME,    "mailNickname" ),
 
 	/* email addresses */
 	STRING_PROP   (E_CONTACT_EMAIL_1,     "mail" ),
@@ -226,9 +228,13 @@ gal_connect (EBookBackendGAL *bl)
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
 
 	g_object_ref (blpriv->gc);
+	g_mutex_lock (blpriv->ldap_lock);
 	blpriv->ldap = e2k_global_catalog_get_ldap (blpriv->gc, NULL);
-	if (!blpriv->ldap)
+	if (!blpriv->ldap) {
+		g_mutex_unlock (blpriv->ldap_lock);
 		return GNOME_Evolution_Addressbook_RepositoryOffline;
+	}
+	g_mutex_unlock (blpriv->ldap_lock);
 
 	blpriv->connected = TRUE;
 	e_book_backend_set_is_loaded (E_BOOK_BACKEND (bl), TRUE);
@@ -238,15 +244,30 @@ gal_connect (EBookBackendGAL *bl)
 static gboolean
 gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 {
+	g_mutex_lock (bl->priv->ldap_lock);
+	if (!bl->priv->ldap) {
+		g_mutex_unlock (bl->priv->ldap_lock);
+		return FALSE;
+	}
+	g_mutex_unlock (bl->priv->ldap_lock);
+
 	/* we need to reconnect if we were previously connected */
 	if (bl->priv->connected && ldap_status == LDAP_SERVER_DOWN) {
+		g_mutex_lock (bl->priv->ldap_lock);
+
 		book_view_notify_status (book_view, _("Reconnecting to LDAP server..."));
 		if (bl->priv->ldap)
 			ldap_unbind (bl->priv->ldap);
 		bl->priv->ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
 		book_view_notify_status (book_view, "");
 
-		return (bl->priv->ldap != NULL);
+		if (bl->priv->ldap != NULL) {
+			g_mutex_unlock (bl->priv->ldap_lock);
+			return TRUE;
+		} else {
+			g_mutex_unlock (bl->priv->ldap_lock);
+			return FALSE;
+		}
 	}
 	else {
 		return FALSE;
@@ -298,8 +319,10 @@ ldap_op_finished (LDAPOp *op)
 	g_hash_table_remove (bl->priv->id_to_op, &op->id);
 
 	/* should handle errors here */
+	g_mutex_lock (bl->priv->ldap_lock);
 	if (bl->priv->ldap)
 		ldap_abandon (bl->priv->ldap, op->id);
+	g_mutex_unlock (bl->priv->ldap_lock);
 
 	op->dtor (op);
 
@@ -375,18 +398,25 @@ get_contact_handler (LDAPOp *op, LDAPMessage *res)
 	EBookBackendGAL *bl = E_BOOK_BACKEND_GAL (op->backend);
 	int msg_type;
 
+	g_mutex_lock (bl->priv->ldap_lock);
 	if (!bl->priv->ldap) {
-		e_data_book_respond_get_contact (op->book, op->opid, GNOME_Evolution_Addressbook_OtherError, "");
+		g_mutex_unlock (bl->priv->ldap_lock);
+		e_data_book_respond_get_contact (op->book, op->opid, 
+					GNOME_Evolution_Addressbook_OtherError, "");
 		ldap_op_finished (op);
 		return;
 	}
+	g_mutex_unlock (bl->priv->ldap_lock);
 
 	/* the msg_type will be either SEARCH_ENTRY (if we're
 	   successful) or SEARCH_RESULT (if we're not), so we finish
 	   the op after either */
 	msg_type = ldap_msgtype (res);
 	if (msg_type == LDAP_RES_SEARCH_ENTRY) {
+		g_mutex_lock (bl->priv->ldap_lock);
 		LDAPMessage *e = ldap_first_entry(bl->priv->ldap, res);
+		g_mutex_unlock (bl->priv->ldap_lock);
+
 		EContact *contact;
 		char *vcard;
 
@@ -413,8 +443,12 @@ get_contact_handler (LDAPOp *op, LDAPMessage *res)
 	else if (msg_type == LDAP_RES_SEARCH_RESULT) {
 		char *ldap_error_msg;
 		int ldap_error;
+	
+		g_mutex_lock (bl->priv->ldap_lock);
 		ldap_parse_result (bl->priv->ldap, res, &ldap_error,
 				   NULL, &ldap_error_msg, NULL, NULL, 0);
+		g_mutex_unlock (bl->priv->ldap_lock);
+
 		if (ldap_error != LDAP_SUCCESS) {
 			g_warning ("get_contact_handler: %02X (%s), additional info: %s",
 				   ldap_error,
@@ -486,21 +520,27 @@ get_contact (EBookBackend *backend,
 		return;
 
 	case GNOME_Evolution_Addressbook_MODE_REMOTE : 	
+		g_mutex_lock (bl->priv->ldap_lock);
 		if (!ldap) {
+			g_mutex_unlock (bl->priv->ldap_lock);
 			e_data_book_respond_get_contact (book, opid, GNOME_Evolution_Addressbook_OtherError, "");
 			return;
 		}
+		g_mutex_unlock (bl->priv->ldap_lock);
+
 		get_contact_op = g_new0 (LDAPGetContactOp, 1);
 
 		book_view = find_book_view (bl);
 
 		do {	
-			ldap_error = ldap_search_ext (bl->priv->ldap, id,
+			g_mutex_lock (bl->priv->ldap_lock);
+			ldap_error = ldap_search_ext (ldap, id,
 						      LDAP_SCOPE_BASE,
 						      "(objectclass=*)",
 						      search_attrs, 0, NULL, NULL,
 						      NULL, /* XXX timeout */
 						      1, &get_contact_msgid);
+			g_mutex_unlock (bl->priv->ldap_lock);
 		} while (gal_reconnect (bl, book_view, ldap_error));
 
 		if (ldap_error == LDAP_SUCCESS) {
@@ -532,15 +572,20 @@ contact_list_handler (LDAPOp *op, LDAPMessage *res)
 	LDAPMessage *e;
 	int msg_type;
 
+	g_mutex_lock (bl->priv->ldap_lock);
 	if (!ldap) {
+		g_mutex_unlock (bl->priv->ldap_lock);
 		e_data_book_respond_get_contact_list (op->book, op->opid, GNOME_Evolution_Addressbook_OtherError, NULL);
 		ldap_op_finished (op);
 		return;
 	}
+	g_mutex_unlock (bl->priv->ldap_lock);
 
 	msg_type = ldap_msgtype (res);
 	if (msg_type == LDAP_RES_SEARCH_ENTRY) {
+		g_mutex_lock (bl->priv->ldap_lock);	
 		e = ldap_first_entry(ldap, res);
+		g_mutex_unlock (bl->priv->ldap_lock);
 
 		while (NULL != e) {
 			EContact *contact = build_contact_from_entry (bl, e, NULL);
@@ -553,15 +598,19 @@ contact_list_handler (LDAPOp *op, LDAPMessage *res)
 
 			g_free (vcard);
 			g_object_unref (contact);
+			g_mutex_lock (bl->priv->ldap_lock);
 			e = ldap_next_entry(ldap, e);
+			g_mutex_unlock (bl->priv->ldap_lock);
 		}
 	}
 	else if (msg_type == LDAP_RES_SEARCH_RESULT) {
 		char *ldap_error_msg;
 		int ldap_error;
 
+		g_mutex_lock (bl->priv->ldap_lock);
 		ldap_parse_result (ldap, res, &ldap_error,
 				   NULL, &ldap_error_msg, NULL, NULL, 0);
+		g_mutex_unlock (bl->priv->ldap_lock);
 		if (ldap_error != LDAP_SUCCESS) {
 			g_warning ("contact_list_handler: %02X (%s), additional info: %s",
 				   ldap_error,
@@ -655,10 +704,13 @@ get_contact_list (EBookBackend *backend,
 		return;
 		
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
+		g_mutex_lock (bl->priv->ldap_lock);
 		if (!ldap) {
+			g_mutex_unlock (bl->priv->ldap_lock);
 			e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_OtherError, NULL);
 			return;
 		}
+		g_mutex_unlock (bl->priv->ldap_lock);
 
 		contact_list_op = g_new0 (LDAPGetContactListOp, 1);
 		book_view = find_book_view (bl);
@@ -672,12 +724,14 @@ get_contact_list (EBookBackend *backend,
 		d(printf ("getting contact list with filter: %s\n", ldap_query));
 
 		do {	
+			g_mutex_lock (bl->priv->ldap_lock);
 			ldap_error = ldap_search_ext (bl->priv->ldap, LDAP_ROOT_DSE,
 						      LDAP_SCOPE_SUBTREE,
 						      ldap_query,
 						      search_attrs, 0, NULL, NULL,
 						      NULL, /* XXX timeout */
 						      LDAP_NO_LIMIT, &contact_list_msgid);
+			g_mutex_unlock (bl->priv->ldap_lock);
 		} while (gal_reconnect (bl, book_view, ldap_error));
 
 		g_free (ldap_query);
@@ -902,11 +956,18 @@ func_is_or_begins_with(ESExp *f, int argc, ESExpResult **argv, gboolean exact)
 	str = rfc2254_escape(argv[1]->value.string);
 	star = exact ? "" : "*";
 
-	if (!exact && strlen (str) == 0) {
+	if (!exact && strlen (str) == 0 && strcmp(propname, "file_as")) {
 		/* Can't do (beginswith FIELD "") */
 		return e_sexp_result_new(f, ESEXP_RES_UNDEFINED);
 	}
 
+	/* We use the query "(beginswith fileas "")" while building cache for
+	 * GAL offline, where we try to retrive all the contacts and store it 
+	 * locally. Retrieving *all* the contacts may not be possible in case 
+	 * of large number of contacts and huge data, (for the same reason
+	 * we don't support empty queries in GAL when online.) In such cases 
+	 * cache may not be complete.
+	 */
 	if (!strcmp(propname, "file_as")) {
 		filter = g_strdup_printf("(displayName=%s%s)", str, star);
 		goto done;
@@ -941,11 +1002,11 @@ func_is_or_begins_with(ESExp *f, int argc, ESExpResult **argv, gboolean exact)
 			g_free (first);
 			g_free (last);
 		} else {
-			filter = g_strdup_printf("(|(displayName=%s%s)(sn=%s%s)(givenName=%s%s))",
+			filter = g_strdup_printf("(|(displayName=%s%s)(sn=%s%s)(givenName=%s%s)(mailNickname=%s%s))",
 						 str, star, str, star,
-						 str, star);
+						 str, star, str, star);
 		}
-	} else
+	} else 
 		filter = g_strdup_printf("(%s=%s%s)", ldap_attr, str, star);
 
  done:
@@ -1077,18 +1138,26 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 	char *attr;
 	BerElement *ber = NULL;
 
+	g_mutex_lock (bl->priv->ldap_lock);
 	dn = ldap_get_dn(ldap, e);
+	g_mutex_unlock (bl->priv->ldap_lock);
+
 	e_contact_set (contact, E_CONTACT_UID, dn);
 	ldap_memfree (dn);
 
-	for (attr = ldap_first_attribute (ldap, e, &ber); attr;
-	     attr = ldap_next_attribute (ldap, e, ber)) {
+	g_mutex_lock (bl->priv->ldap_lock);
+	attr = ldap_first_attribute (ldap, e, &ber);
+	g_mutex_unlock (bl->priv->ldap_lock);
+
+	while (attr) {
 		int i;
 		struct prop_info *info = NULL;
 		char **values;
 
 		if (existing_objectclasses && !g_ascii_strcasecmp (attr, "objectclass")) {
+			g_mutex_lock (bl->priv->ldap_lock);
 			values = ldap_get_values (ldap, e, attr);
+			g_mutex_unlock (bl->priv->ldap_lock);
 			for (i = 0; values[i]; i ++)
 				*existing_objectclasses = g_list_append (*existing_objectclasses, g_strdup (values[i]));
 
@@ -1106,7 +1175,9 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 
 			if (info) {
 				if (1) {
+					g_mutex_lock (bl->priv->ldap_lock);
 					values = ldap_get_values (ldap, e, attr);
+					g_mutex_unlock (bl->priv->ldap_lock);
 
 					if (values) {
 						if (info->prop_type & PROP_TYPE_STRING) {
@@ -1128,6 +1199,9 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 		}
 
 		ldap_memfree (attr);
+		g_mutex_lock (bl->priv->ldap_lock);
+		attr = ldap_next_attribute (ldap, e, ber);
+		g_mutex_unlock (bl->priv->ldap_lock);
 	}
 
 	if (ber)
@@ -1144,10 +1218,13 @@ poll_ldap (EBookBackendGAL *bl)
 	LDAPMessage    *res;
 	struct timeval timeout;
 
+	g_mutex_lock (bl->priv->ldap_lock);
 	if (!ldap) {
+		g_mutex_unlock (bl->priv->ldap_lock);
 		bl->priv->poll_timeout = -1;
 		return FALSE;
 	}
+	g_mutex_unlock (bl->priv->ldap_lock);
 
 	if (!bl->priv->active_ops) {
 		g_warning ("poll_ldap being called for backend with no active operations");
@@ -1158,7 +1235,9 @@ poll_ldap (EBookBackendGAL *bl)
 	timeout.tv_sec = 0;
 	timeout.tv_usec = LDAP_RESULT_TIMEOUT_MILLIS * 1000;
 
+	g_mutex_lock (bl->priv->ldap_lock);
 	rc = ldap_result (ldap, LDAP_RES_ANY, 0, &timeout, &res);
+	g_mutex_unlock (bl->priv->ldap_lock);
 	if (rc != 0) {/* rc == 0 means timeout exceeded */
 		if (rc == -1) {
 			EDataBookView *book_view = find_book_view (bl);
@@ -1207,11 +1286,14 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 
 	d(printf ("ldap_search_handler (%p)\n", view));
 
+	g_mutex_lock (bl->priv->ldap_lock);
 	if (!ldap) {
+		g_mutex_unlock (bl->priv->ldap_lock);
 		e_data_book_view_notify_complete (view, GNOME_Evolution_Addressbook_OtherError);
 		ldap_op_finished (op);
 		return;
 	}
+	g_mutex_unlock (bl->priv->ldap_lock);
 
 	if (!search_op->notified_receiving_results) {
 		search_op->notified_receiving_results = TRUE;
@@ -1220,7 +1302,9 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 
 	msg_type = ldap_msgtype (res);
 	if (msg_type == LDAP_RES_SEARCH_ENTRY) {
+		g_mutex_lock (bl->priv->ldap_lock);
 		e = ldap_first_entry(ldap, res);
+		g_mutex_unlock (bl->priv->ldap_lock);
 
 		while (NULL != e) {
 			EContact *contact = build_contact_from_entry (bl, e, NULL);
@@ -1229,15 +1313,19 @@ ldap_search_handler (LDAPOp *op, LDAPMessage *res)
 
 			g_object_unref (contact);
 
+			g_mutex_lock (bl->priv->ldap_lock);
 			e = ldap_next_entry(ldap, e);
+			g_mutex_unlock (bl->priv->ldap_lock);
 		}
 	}
 	else if (msg_type == LDAP_RES_SEARCH_RESULT) {
 		char *ldap_error_msg;
 		int ldap_error;
 
+		g_mutex_lock (bl->priv->ldap_lock);
 		ldap_parse_result (ldap, res, &ldap_error,
 				   NULL, &ldap_error_msg, NULL, NULL, 0);
+		g_mutex_unlock (bl->priv->ldap_lock);
 		if (ldap_error != LDAP_SUCCESS) {
 			g_warning ("ldap_search_handler: %02X (%s), additional info: %s",
 				   ldap_error,
@@ -1314,11 +1402,14 @@ start_book_view (EBookBackend  *backend,
 		return;
 		
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
+		g_mutex_lock (bl->priv->ldap_lock);
 		if (!bl->priv->ldap) {
+			g_mutex_unlock (bl->priv->ldap_lock);
 			e_data_book_view_notify_complete (view,
 							  GNOME_Evolution_Addressbook_InvalidQuery);
 			return;
 		}
+		g_mutex_unlock (bl->priv->ldap_lock);
 
 		/* we start at 1 so the user sees stuff as it appears and we
 		   aren't left waiting for more cards to show up, if the
@@ -1333,6 +1424,7 @@ start_book_view (EBookBackend  *backend,
 
 		status = build_query (bl, e_data_book_view_get_card_query (view),
 				      &ldap_query);
+
 		if (status != GNOME_Evolution_Addressbook_Success || !ldap_query) {
 			e_data_book_view_notify_complete (view, status);
 			if (ldap_query)
@@ -1343,6 +1435,7 @@ start_book_view (EBookBackend  *backend,
 		do {
 			book_view_notify_status (view, _("Searching..."));
 
+			g_mutex_lock (bl->priv->ldap_lock);
 			ldap_err = ldap_search_ext (bl->priv->ldap, LDAP_ROOT_DSE,
 						    LDAP_SCOPE_SUBTREE,
 						    ldap_query,
@@ -1352,6 +1445,7 @@ start_book_view (EBookBackend  *backend,
 						    NULL, /* XXX timeout */
 						    view_limit,
 						    &search_msgid);
+			g_mutex_unlock (bl->priv->ldap_lock);
 		} while (gal_reconnect (bl, view, ldap_err));
 
 		g_free (ldap_query);
@@ -1414,26 +1508,33 @@ generate_cache_handler (LDAPOp *op, LDAPMessage *res)
 	LDAPMessage *e;
 	gint msg_type;
 
+	g_mutex_lock (bl->priv->ldap_lock);
 	if (!ldap) {
+		g_mutex_unlock (bl->priv->ldap_lock);
 		ldap_op_finished (op);
 		return;
 	}
+	g_mutex_unlock (bl->priv->ldap_lock);
 
 	msg_type = ldap_msgtype (res);
 	if (msg_type == LDAP_RES_SEARCH_ENTRY) {
+		g_mutex_lock (bl->priv->ldap_lock);
 		e = ldap_first_entry(ldap, res);
+		g_mutex_unlock (bl->priv->ldap_lock);
 
 		while (e != NULL) {
 			EContact *contact = build_contact_from_entry (bl, e, NULL);
-			contact_list_op->contacts = g_list_prepend (contact_list_op->contacts, contact);
 
+			contact_list_op->contacts = g_list_prepend (contact_list_op->contacts, contact);
+			g_mutex_lock (bl->priv->ldap_lock);
 			e = ldap_next_entry(ldap, e);
+			g_mutex_unlock (bl->priv->ldap_lock);
 		}
-	} else {
+	} else if (msg_type == LDAP_RES_SEARCH_RESULT) {
 		GList *l;
 
 		e_file_cache_clean (E_FILE_CACHE (bl->priv->cache));
-
+		
 		for (l = contact_list_op->contacts; l; l = g_list_next (l)) {
 			EContact *contact = l->data;
 			e_book_backend_cache_add_contact (bl->priv->cache, contact);
@@ -1469,21 +1570,26 @@ generate_cache (EBookBackendGAL *book_backend_gal)
 
 	priv = book_backend_gal->priv;
 
+	g_mutex_lock (priv->ldap_lock);
 	if (!priv->ldap) {
+		g_mutex_unlock (priv->ldap_lock);
 		g_free (contact_list_op);
 		return;
 	}
+	g_mutex_unlock (priv->ldap_lock);
 
 	build_query (book_backend_gal, 
 		     "(beginswith \"file_as\" \"\")", &ldap_query);
 
 	do {	
+		g_mutex_lock (priv->ldap_lock);
 		ldap_error = ldap_search_ext (priv->ldap, LDAP_ROOT_DSE,
 					      LDAP_SCOPE_SUBTREE,
 					      ldap_query,
 					      NULL, 0, NULL, NULL,
 					      NULL, /* XXX timeout */
 					      LDAP_NO_LIMIT, &contact_list_msgid);
+		g_mutex_unlock (priv->ldap_lock);
 	} while (gal_reconnect (book_backend_gal, NULL, ldap_error));
 
 	g_free (ldap_query);
@@ -1518,8 +1624,10 @@ ldap_cancel_op(void *key, void *value, void *data)
 	LDAPOp *op = value;
 
 	/* ignore errors, its only best effort? */
+	g_mutex_lock (bl->priv->ldap_lock);
 	if (bl->priv->ldap)
 		ldap_abandon_ext (bl->priv->ldap, op->id, NULL, NULL);
+	g_mutex_unlock (bl->priv->ldap_lock);
 }
 
 static GNOME_Evolution_Addressbook_CallStatus
@@ -1561,8 +1669,8 @@ set_mode (EBookBackend *backend, int mode)
 			}
 		} else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
 			if (exchange_account_connect (bepriv->account)) {
-				e_book_backend_set_is_writable (backend, TRUE);
-				e_book_backend_notify_writable (backend, TRUE);
+				e_book_backend_set_is_writable (backend, FALSE);
+				e_book_backend_notify_writable (backend, FALSE);
 				e_book_backend_notify_connection_status (backend, TRUE);
 
 				if (e_book_backend_is_loaded (backend)) {
@@ -1712,7 +1820,11 @@ e_book_backend_gal_new (void)
 static gboolean
 call_dtor (int msgid, LDAPOp *op, gpointer data)
 {
-	ldap_abandon (E_BOOK_BACKEND_GAL(op->backend)->priv->ldap, op->id);
+	EBookBackendGAL *bl = E_BOOK_BACKEND_GAL (op->backend);
+
+	g_mutex_lock (bl->priv->ldap_lock);
+	ldap_abandon (bl->priv->ldap, op->id);
+	g_mutex_unlock (bl->priv->ldap_lock);
 
 	op->dtor (op);
 
@@ -1736,13 +1848,19 @@ dispose (GObject *object)
 			g_source_remove (bl->priv->poll_timeout);
 		}
 
+		g_mutex_lock (bl->priv->ldap_lock);
 		if (bl->priv->ldap)
 			ldap_unbind (bl->priv->ldap);
+		g_mutex_unlock (bl->priv->ldap_lock);
+
 		if (bl->priv->gc)
 			g_object_unref (bl->priv->gc);
 		if (bl->priv->cache)
 			g_object_unref (bl->priv->cache);
 
+		if (bl->priv->ldap_lock)
+			g_mutex_free (bl->priv->ldap_lock);
+
 		g_free (bl->priv->gal_uri);
 		g_free (bl->priv);
 		bl->priv = NULL;
@@ -1805,6 +1923,7 @@ init (EBookBackendGAL *backend)
 
 	priv->id_to_op         	     = g_hash_table_new (g_int_hash, g_int_equal);
 	priv->poll_timeout     	     = -1;
+	priv->ldap_lock		     = g_mutex_new ();
 
 	g_static_rec_mutex_init (&priv->op_hash_mutex);
 
