@@ -1023,7 +1023,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		dt.value->is_utc = FALSE;
 		dt.value->hour = dt.value->minute = dt.value->second = 0;
 		dt.value->zone = zone;
-		dt.tzid = icaltimezone_get_tzid (zone);
+		
+		g_free ((char *)dt.tzid);
+		dt.tzid = g_strdup (icaltimezone_get_tzid (zone));
 		e_cal_component_set_dtstart (real_ecomp, &dt);
 		e_cal_component_free_datetime (&dt);
 
@@ -1032,7 +1034,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		dt.value->is_utc = FALSE;
 		dt.value->hour = dt.value->minute = dt.value->second = 0;
 		dt.value->zone = zone;
-		dt.tzid = icaltimezone_get_tzid (zone);
+
+		g_free ((char *)dt.tzid);
+		dt.tzid = g_strdup (icaltimezone_get_tzid (zone));
 		e_cal_component_set_dtend (real_ecomp, &dt);
 	}
 	e_cal_component_free_datetime (&dt);
@@ -1207,9 +1211,9 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 		
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp));
-	if(object)
-		*object = e_cal_component_get_as_string (comp);
-	
+	if (old_object)
+		*old_object = e_cal_component_get_as_string (comp);
+
 	switch (mod) {
 		
 		case CALOBJ_MOD_ALL:
@@ -1223,6 +1227,7 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 						return GNOME_Evolution_Calendar_Success;
 				}
 			}
+			*object = NULL;
 			break;
 		case CALOBJ_MOD_THIS:
 			/*remove_instance and modify */
@@ -1275,7 +1280,8 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 	for (l = comps; l; l= l->next) {
 		const char *uid, *rid;
 		char *calobj;
-		
+		char *object = NULL;
+			
 		subcomp = l->data;
 		
 		comp = e_cal_component_new ();
@@ -1321,8 +1327,18 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 
 		case ICAL_METHOD_CANCEL:
 			calobj = (char *) icalcomponent_as_ical_string (subcomp);
-			status = remove_object (backend, cal, uid, rid, CALOBJ_MOD_THIS, &calobj, NULL);
-			e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, calobj, NULL);
+			
+			if (rid)
+				status = remove_object (backend, cal, uid, rid, CALOBJ_MOD_THIS, &calobj, &object);
+			else
+				status = remove_object (backend, cal, uid, NULL, CALOBJ_MOD_ALL, &calobj, &object);
+
+			if (status == GNOME_Evolution_Calendar_Success) 
+				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, calobj, NULL);
+			if (object) {
+				g_free (object);
+				object = NULL;
+			}
 			break;
 		default:
 			status = GNOME_Evolution_Calendar_UnsupportedMethod;
@@ -1453,6 +1469,8 @@ book_resource (ECalBackendExchange *cbex,
 	ecomp = get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbex), uid);
 												 
 	if (method == ICAL_METHOD_CANCEL) {
+		char *object = NULL;
+			
 		/* g_object_unref (comp); */
 		/* If there is nothing to cancel, we're good */
 		if (!ecomp) {
@@ -1474,9 +1492,10 @@ book_resource (ECalBackendExchange *cbex,
 		e_cal_component_set_transparency (E_CAL_COMPONENT (comp), E_CAL_COMPONENT_TRANSP_TRANSPARENT);
 		calobj = (char *) e_cal_component_get_as_string (comp);
 		rid = (char *) e_cal_component_get_recurid_as_string (comp);
-		status = remove_object (E_CAL_BACKEND_SYNC (cbex), cal, uid, rid, CALOBJ_MOD_THIS, &calobj, NULL);
+		status = remove_object (E_CAL_BACKEND_SYNC (cbex), cal, uid, rid, CALOBJ_MOD_THIS, &calobj, &object);
 		e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbex), uid, calobj, NULL);
 		g_free (calobj); 
+		g_free (object);
 	} else {
 		/* Check that the new appointment doesn't conflict with any
 		 * existing appointment.
