@@ -296,6 +296,7 @@ get_ctx_auth_handler (SoupMessage *msg, gpointer user_data)
 	const char *challenge_hdr;
 	GByteArray *challenge;
 
+	ac->saw_ntlm = ac->saw_basic = FALSE;
 	headers = soup_message_get_header_list (msg->response_headers,
 						"WWW-Authenticate");
 	while (headers) {
@@ -316,6 +317,7 @@ get_ctx_auth_handler (SoupMessage *msg, gpointer user_data)
 					       ac->nt_domain ? NULL : &ac->nt_domain,
 					       ac->w2k_domain ? NULL : &ac->w2k_domain);
 			g_byte_array_free (challenge, TRUE);
+			ac->saw_ntlm = TRUE;
 			return;
 		}
 
@@ -1389,13 +1391,15 @@ e2k_autoconfig_lookup_option (const char *option)
 }
 
 static gboolean 
-validate (const char *owa_url, char *user, char *password, char **host, char **ad_server)
+validate (const char *owa_url, char *user, char *password, ExchangeParams *exchange_params)
 {
 	E2kAutoconfig *ac;
 	E2kOperation op;        /* FIXME */
 	E2kAutoconfigResult result;
+	E2kUri *euri;
 	gboolean valid = FALSE;
 	const char *old, *new;
+	char *path, *mailbox;
 
 	ac = e2k_autoconfig_new (owa_url, user, password, 
 				 E2K_AUTOCONFIG_USE_EITHER);
@@ -1405,9 +1409,50 @@ validate (const char *owa_url, char *user, char *password, char **host, char **a
 	// e2k_autoconfig_set_gc_server (ac, NULL, -1);
 	result = e2k_autoconfig_check_exchange (ac, &op);
 
+	if (result == E2K_AUTOCONFIG_OK) {
+		/*
+		 * On error code 403 and SSL seen in server response 
+		 * e2k_autoconfig_get_context() tries to
+		 * connect using https if owa url has http and vice versa.
+		 * And also re-sets the owa_uri in E2kAutoconfig.
+		 * So even if the uri is incorrect, 
+		 * e2k_autoconfig_check_exchange() will return success.
+		 * In this case of account set up, owa_url paramter will still
+		 * have wrong url entered, and we throw the error, instead of
+		 * going ahead with account setup and failing later. 
+		 */
+		if (g_str_has_prefix (ac->owa_uri, "http:")) {
+		    if (!g_str_has_prefix (owa_url, "http:"))
+			result = E2K_AUTOCONFIG_CANT_CONNECT;
+		}
+		else if (!g_str_has_prefix (owa_url, "https:"))
+			result = E2K_AUTOCONFIG_CANT_CONNECT;
+	}
+
 	if (result == E2K_AUTOCONFIG_OK) {
 		result = e2k_autoconfig_check_global_catalog (ac, &op);
 		e2k_operation_free (&op);
+		
+		/* find mailbox and owa_path values */	
+		euri = e2k_uri_new (ac->home_uri);
+		path = g_strdup (euri->path + 1);
+		e2k_uri_free (euri);
+		mailbox = strrchr (path, '/');
+		if (mailbox && !mailbox[1]) {
+			*mailbox = '\0';
+			mailbox = strrchr (path, '/');
+		}
+		if (mailbox)
+			*mailbox++ = '\0';
+
+		exchange_params->mailbox  = g_strdup (mailbox);
+		exchange_params->owa_path = g_strdup_printf ("%s%s", "/", path);
+		g_free (path);
+		exchange_params->host = ac->pf_server;
+		if (ac->gc_server) 
+			exchange_params->ad_server = ac->gc_server;
+		exchange_params->is_ntlm = ac->saw_ntlm;
+
 		valid = TRUE;
 	}
 	else {
@@ -1504,16 +1549,13 @@ validate (const char *owa_url, char *user, char *password, char **host, char **a
 		}
 	}
 
-	*host = ac->pf_server;
-	if (ac->gc_server) 
-		*ad_server = ac->gc_server;
 
 	return valid;
 }
 
 gboolean
-e2k_validate_user (const char *owa_url, char *user, char **host, 
-		   char **ad_server, gboolean *remember_password)
+e2k_validate_user (const char *owa_url, char *user,
+		   ExchangeParams *exchange_params, gboolean *remember_password)
 {
 	gboolean valid = FALSE, remember=FALSE;
 	char *key, *password, *prompt;
@@ -1527,7 +1569,7 @@ e2k_validate_user (const char *owa_url, char *user, char **host,
 					E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
 					&remember, NULL);
 		if (password) {
-			valid = validate (owa_url, user, password, host, ad_server);
+			valid = validate (owa_url, user, password, exchange_params);
 			if (valid) {
 				/* generate the proper key once the host name 
 				 * is read and remember password temporarily, 
@@ -1539,7 +1581,7 @@ e2k_validate_user (const char *owa_url, char *user, char **host,
 				*remember_password = remember;
 				g_free (key);
 				key = g_strdup_printf ("%s//%s@%s", 
-						       "exchange:", user, *host);
+						       "exchange:", user, exchange_params->host);
 				e_passwords_add_password (key, password);
 				e_passwords_remember_password ("Exchange", key);
 			}
