@@ -58,7 +58,8 @@ static ExchangeAccountFolderResult create_folder (ExchangeHierarchy *hier,
 static ExchangeAccountFolderResult remove_folder (ExchangeHierarchy *hier,
 						  EFolder *folder);
 static ExchangeAccountFolderResult scan_subtree (ExchangeHierarchy *hier,
-						 EFolder *folder);
+						 EFolder *folder,
+						 gboolean offline);
 static void finalize (GObject *object);
 
 static void
@@ -335,10 +336,6 @@ create_folder (ExchangeHierarchy *hier, EFolder *parent,
 static ExchangeAccountFolderResult
 remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 {
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
-	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type, *physical_uri;
 
 	/* Temp Fix for remove fav folders. see #59168 */
@@ -347,31 +344,19 @@ remove_folder (ExchangeHierarchy *hier, EFolder *folder)
         physical_uri = e_folder_get_physical_uri (folder);
 
         if (strcmp (folder_type, "calendar") == 0) {
-                cal_source_list = e_source_list_new_for_gconf (
-                                        gconf_client_get_default (),
-                                        conf_key_cal);
-                remove_esource (hier->account, conf_key_cal, physical_uri,
-                                &cal_source_list, FALSE, FALSE);
-                e_source_list_sync (cal_source_list, NULL);
-                g_object_unref (cal_source_list);
+                remove_folder_esource (hier->account,
+				       EXCHANGE_CALENDAR_FOLDER,
+				       physical_uri);
         }
         else if (strcmp (folder_type, "tasks") == 0) {
-                task_source_list = e_source_list_new_for_gconf (
-                                        gconf_client_get_default (),
-                                        conf_key_tasks);
-                remove_esource (hier->account, conf_key_tasks, physical_uri,
-                                &task_source_list, FALSE, FALSE);
-                e_source_list_sync (task_source_list, NULL);
-                g_object_unref (task_source_list);
+                remove_folder_esource (hier->account,
+				       EXCHANGE_TASKS_FOLDER,
+				       physical_uri);
         }
         else if (strcmp (folder_type, "contacts") == 0) {
-                cont_source_list = e_source_list_new_for_gconf (
-                                        gconf_client_get_default (),
-                                        conf_key_contacts);
-                remove_esource (hier->account, conf_key_contacts, physical_uri,
-                                        &cont_source_list, FALSE, TRUE);
-                e_source_list_sync (cont_source_list, NULL);
-                g_object_unref (cont_source_list);
+                remove_folder_esource (hier->account,
+				       EXCHANGE_CONTACTS_FOLDER,
+				       physical_uri);
 	}
 
 	if (folder != hier->toplevel)
@@ -384,13 +369,13 @@ remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 }
 
 static ExchangeAccountFolderResult
-scan_subtree (ExchangeHierarchy *hier, EFolder *folder)
+scan_subtree (ExchangeHierarchy *hier, EFolder *folder, gboolean offline)
 {
 	ExchangeAccountFolderResult folder_result;
 
 	check_hide_private (hier);
 
-	folder_result = EXCHANGE_HIERARCHY_CLASS (parent_class)->scan_subtree (hier, folder);
+	folder_result = EXCHANGE_HIERARCHY_CLASS (parent_class)->scan_subtree (hier, folder, offline);
 
 	if (exchange_hierarchy_is_empty (hier))
 		hierarchy_foreign_cleanup (hier);
@@ -435,10 +420,6 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 	const char *folder_type = NULL;
 	const char *physical_uri = NULL;
 	char *new_folder_name;
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
-	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 
 	result =  create_internal (hier, hier->toplevel, folder_name, NULL, folder);
 
@@ -451,50 +432,27 @@ exchange_hierarchy_foreign_add_folder (ExchangeHierarchy *hier,
 
 		if (!(strcmp (folder_type, "calendar")) ||
 		!(strcmp (folder_type, "calendar/public"))) {
-			cal_source_list = e_source_list_new_for_gconf (
-						gconf_client_get_default (),
-						conf_key_cal);
-			add_esource (hier->account,
-				     conf_key_cal,
-				     new_folder_name,
-				     physical_uri,
-				     &cal_source_list,
-				     FALSE);
-			e_source_list_sync (cal_source_list, NULL);
-			g_object_unref (cal_source_list);
-
+			add_folder_esource (hier->account,
+				     	    EXCHANGE_CALENDAR_FOLDER,
+				     	    new_folder_name,
+				    	    physical_uri);
 		}
 		else if (!(strcmp (folder_type, "tasks")) ||
 			 !(strcmp (folder_type, "tasks/public"))) {
-				task_source_list = e_source_list_new_for_gconf (
-						gconf_client_get_default (),
-						conf_key_tasks);
-				add_esource (hier->account,
-				     conf_key_tasks,
-				     new_folder_name,
-				     physical_uri,
-				     &task_source_list,
-				     FALSE);
-				e_source_list_sync (task_source_list, NULL);
-				g_object_unref (task_source_list);
-
+				add_folder_esource (hier->account,
+				     		    EXCHANGE_TASKS_FOLDER,
+				     		    new_folder_name,
+				     		    physical_uri);
 		}
 		else if (!(strcmp (folder_type, "contacts")) ||
 			 !(strcmp (folder_type, "contacts/public")) ||
 			 !(strcmp (folder_type, "contacts/ldap"))) {
-				cont_source_list = e_source_list_new_for_gconf (
-						gconf_client_get_default (),
-						conf_key_contacts);
-
-				add_esource (hier->account,
-				     conf_key_contacts,
-				     new_folder_name,
-				     physical_uri,
-				     &cont_source_list,
-				     TRUE);
-				e_source_list_sync (cont_source_list, NULL);
-				g_object_unref (cont_source_list);
+				add_folder_esource (hier->account,
+				     		    EXCHANGE_CONTACTS_FOLDER,
+				     		    new_folder_name,
+				     		    physical_uri);
 		}
+		g_free (new_folder_name);
 	}
 
 	return result;
@@ -581,6 +539,7 @@ exchange_hierarchy_foreign_new (ExchangeAccount *account,
 	xmlSaveFile (mf_path, doc);
 	g_hash_table_destroy (props);
 	g_free (mf_path);
+	xmlFreeDoc (doc);
 
 	return hier;
 }
